@@ -12,7 +12,7 @@
 #ifndef ZYPP_MEDIA_MEDIAMANAGER_H
 #define ZYPP_MEDIA_MEDIAMANAGER_H
 
-#include <zypp/APIConfig.h>
+#include <zypp-core/Globals.h>
 #include <zypp/base/NonCopyable.h>
 #include <zypp/base/PtrTypes.h>
 #include <zypp/Pathname.h>
@@ -29,7 +29,7 @@ namespace zypp::media
 {
   class MediaHandler;
 
-  typedef unsigned int  MediaNr;
+  using MediaNr = unsigned int;
 
 
   ///////////////////////////////////////////////////////////////////
@@ -44,7 +44,7 @@ namespace zypp::media
   /**
    * Interface to implement a media verifier.
    */
-  class MediaVerifierBase //: private zypp::NonCopyable
+  class ZYPP_TESTS MediaVerifierBase //: private zypp::NonCopyable
   {
   public:
     MediaVerifierBase()
@@ -104,7 +104,7 @@ namespace zypp::media
   /**
    * A shared reference to the MediaVerifier implementation.
    */
-  typedef zypp::RW_pointer<MediaVerifierBase> MediaVerifierRef;
+  using MediaVerifierRef = zypp::RW_pointer<MediaVerifierBase>;
 
 
   ///////////////////////////////////////////////////////////////////
@@ -450,7 +450,7 @@ namespace zypp::media
    *   - \c script<->libzypp communication:
    *     - \TODO to be documented.
    */
-  class MediaManager: private zypp::base::NonCopyable
+  class ZYPP_API MediaManager: private zypp::base::NonCopyable
   {
   public:
     /**
@@ -818,7 +818,7 @@ namespace zypp::media
      * FIXME: see MediaAccess class.
      */
     bool doesFileExist(MediaAccessId  accessId,
-                       const Pathname & filename ) const;
+                       const Pathname & filename ) const ZYPP_TESTS;
 
     /**
      * Fill in a vector of detected ejectable devices and the index of the
